import logging
from dataclasses import asdict
from dataclasses import dataclass
from dataclasses import field
from pathlib import Path
from threading import Semaphore
from threading import Thread
from typing import Dict
from typing import List
from typing import Optional

from mkdocs.config.defaults import MkDocsConfig

from mkdocs_publisher import _utils
from mkdocs_publisher.minifier.config import MinifierConfig

log = logging.getLogger("mkdocs.plugins.publisher.minifier")


@dataclass
class CachedFile:
    original_file_hash: str = field(default="")
    original_file_path: Path = field(default_factory=lambda: Path(""))
    cached_file_name: Path = field(default_factory=lambda: Path(""))

    def __init__(
        self,
        original_file_hash: str = "",
        original_file_path: str = "",
        cached_file_name: str = "",
    ):
        self.original_file_hash = original_file_hash
        self.original_file_path = Path(original_file_path)
        self.cached_file_name = Path(cached_file_name)

    def based_on(self, file: Path, directory: Path):
        self.original_file_path = file
        self.original_file_hash = _utils.calculate_file_hash(file=directory / file)
        self.cached_file_name = _utils.get_hashed_file_name(file=file)

    @property
    def as_dict(self) -> dict:
        return asdict(self)


class BaseMinifier:
    extensions: List[str]

    def __init__(
        self,
        plugin_config: MinifierConfig,
        mkdocs_config: MkDocsConfig,
        cached_files: Dict[str, CachedFile],
    ):
        self._plugin_config: MinifierConfig = plugin_config
        self._mkdocs_config: MkDocsConfig = mkdocs_config
        self._cached_files: Dict[str, CachedFile] = cached_files

    def minifier(self, cached_file: CachedFile) -> Optional[CachedFile]:
        raise NotImplementedError

    def __call__(self):
        log.info(f"Minifying {self.extensions} files")

        if self._plugin_config.threads < 1:
            log.warning("Number of 'threads' cannot be smaller than 1 (changing to default 8")
            self._plugin_config.threads = 1

        semaphore = Semaphore(self._plugin_config.threads)
        threads = []

        # TODO: add possibility to exclude some files
        for file in _utils.list_files(
            directory=Path(self._mkdocs_config.site_dir),
            extensions=self.extensions,
            relative_to=Path(self._mkdocs_config.site_dir),
        ):
            semaphore.acquire()
            thread = Thread(
                target=self._minify_with_cache,
                kwargs={
                    "file": file,
                    "semaphore": semaphore,
                },
            )
            thread.start()
            threads.append(thread)

        # Wait for all threads to be finished
        for thread in threads:
            thread.join()

    def _is_new_smaller(self, cached_file: CachedFile) -> Optional[CachedFile]:
        old_file = self._mkdocs_config.site_dir / cached_file.original_file_path
        new_file = self._plugin_config.cache_dir / cached_file.cached_file_name
        new_file_size = new_file.stat().st_size
        old_file_size = old_file.stat().st_size
        log.debug(
            f"Minified: '{old_file.relative_to(self._mkdocs_config.site_dir)}' "
            f"(size: {old_file_size} -> {new_file_size})"
        )
        log.debug(
            f"{old_file.relative_to(self._mkdocs_config.site_dir)} -> "
            f"{new_file.relative_to(self._plugin_config.cache_dir)} "
        )
        if new_file_size < old_file_size:
            return cached_file

        log.debug(
            f"Minified file larger than original: "
            f"{cached_file.original_file_path} (removing cached file)"
        )
        new_file.unlink()
        return None

    def _copy_cached_file(self, cached_file: CachedFile):
        original_file = self._mkdocs_config.site_dir / cached_file.original_file_path
        cache_file = self._plugin_config.cache_dir / cached_file.cached_file_name
        try:
            original_file.write_bytes(cache_file.read_bytes())
        except FileNotFoundError as e:
            log.warning(e)

    def _minify_with_cache(self, file: Path, semaphore: Semaphore):
        recreate_file = False
        if str(file) in self._cached_files:
            log.debug(f"{file} is in cache")
            file_hash = _utils.calculate_file_hash(file=(self._mkdocs_config.site_dir / file))
            cached_file = self._cached_files[str(file)]
            if cached_file.original_file_hash == file_hash:
                log.debug(f"{file} hash is equal to one in cache (file: {file_hash})")
                cached_file_name = self._plugin_config.cache_dir / cached_file.cached_file_name
                if cached_file_name.exists():
                    self._copy_cached_file(cached_file=cached_file)
                else:
                    recreate_file = True
            else:
                log.debug(
                    f"{file} hash is not equal to one in cache "
                    f"(file: {file_hash} | cache: {cached_file.original_file_hash})"
                )
                recreate_file = True
        else:
            log.debug(f"{file} is not in cache")
            cached_file = CachedFile()
            cached_file.based_on(file=file, directory=Path(self._mkdocs_config.site_dir))
            recreate_file = True

        if recreate_file:
            log.debug(f"Minifying file: {file} (putting into cache)")
            cached_file = self.minifier(cached_file=cached_file)
            if cached_file:
                cached_file = self._is_new_smaller(cached_file=cached_file)
            if cached_file:
                self._cached_files[str(cached_file.original_file_path)] = cached_file
                self._copy_cached_file(cached_file=cached_file)
        else:
            log.debug(f"File already minified: {file} (retrieving from cache)")
        semaphore.release()



from mkdocs.config import config_options as option
from mkdocs.config.base import Config


class _BlogTranslationConfig(Config):
    teaser_link_text = option.Optional(option.Type(str))
    blog_page_title = option.Optional(option.Type(str))
    blog_navigation_name = option.Optional(option.Type(str))
    recent_blog_posts_navigation_name = option.Optional(option.Type(str))
    archive_page_title = option.Optional(option.Type(str))
    archive_navigation_name = option.Optional(option.Type(str))
    categories_page_title = option.Optional(option.Type(str))
    categories_navigation_name = option.Optional(option.Type(str))
    tags_page_title = option.Optional(option.Type(str))
    tags_navigation_name = option.Optional(option.Type(str))
    newer_posts = option.Optional(option.Type(str))
    older_posts = option.Optional(option.Type(str))


class BlogPluginConfig(Config):
    # General settings
    lang = option.Choice(["en", "pl"], default="en")  # TODO: auto update based on files
    teaser_marker = option.Type(str, default="<!-- more -->")
    posts_per_page = option.Type(int, default=5)
    start_page = option.Type(bool, default=False)
    slug = option.Type(str, default="blog")

    # Directories
    temp_dir = option.Type(str, default=".temp")
    blog_dir = option.Type(str, default="blog")
    archive_subdir = option.Type(str, default="archive")
    categories_subdir = option.Type(str, default="categories")
    tags_subdir = option.Type(str, default="tags")

    # Values that are in lang files and can be overriden
    translation: _BlogTranslationConfig = option.SubConfig(_BlogTranslationConfig)  # type: ignore



import logging
import re
from pathlib import Path
from typing import List
from typing import Optional
from typing import cast

import frontmatter
from mkdocs.config import Config
from mkdocs.config.defaults import MkDocsConfig
from mkdocs.plugins import BasePlugin
from mkdocs.plugins import event_priority
from mkdocs.structure.files import Files
from mkdocs.structure.nav import Link
from mkdocs.structure.nav import Navigation
from mkdocs.structure.nav import Section
from mkdocs.structure.pages import Page

from mkdocs_publisher import _utils
from mkdocs_publisher.auto_nav.config import AutoNavPluginConfig

log = logging.getLogger("mkdocs.plugins.publisher.auto-nav")


class AutoNavPlugin(BasePlugin[AutoNavPluginConfig]):
    def __init__(self):
        from mkdocs_publisher.blog.config import BlogPluginConfig

        self.blog_config: Optional[BlogPluginConfig] = None

    def _read_markdown_title(self, file: Path) -> str:
        with open(file) as file_meta:
            post = frontmatter.load(file_meta)
        if "title" not in post.metadata:
            log.warning(f"File '{file}' doesn't contain 'title' meta data")
        return post.get("title", default=file.stem)

    def _iterate_dir(self, directory: Path, skip_subfiles_of_dir: List[str], relative_to: Path):
        nav = list()
        sorted_files = sorted([f for f in directory.glob("**/*")])

        # Make index.md a first file in give directory
        no_index_files = [f for f in sorted_files if f.name != "index.md"]
        sorted_files = [f for f in sorted_files if f.name == "index.md"]
        sorted_files.extend(no_index_files)

        for file in sorted_files:
            if not any([s for s in skip_subfiles_of_dir if str(file).startswith(s)]):
                if file.is_dir():
                    meta_file = file / self.config.meta_file_name
                    if meta_file.exists():
                        with open(meta_file) as file_meta:
                            post = frontmatter.load(file_meta)
                    else:
                        post = {"title": file.stem}
                    title = post.get("title")
                    subdir_nav = {
                        title: self._iterate_dir(
                            directory=file,
                            skip_subfiles_of_dir=skip_subfiles_of_dir,
                            relative_to=relative_to,
                        )
                    }
                    if list(subdir_nav.values())[0]:
                        nav.append(subdir_nav)
                elif (
                    file.is_file()
                    and len(file.relative_to(directory).parents) == 1
                    and file.suffix == ".md"
                ):
                    if Path(file).parts[-1] != self.config.meta_file_name:
                        title = self._read_markdown_title(file=file)
                        nav.append({title: str(file.relative_to(relative_to))})
                else:
                    log.debug(f"{file} is not .md or dir and will not be added to navigation")
            elif file.suffix == ".md" and not any(
                [p for p in [str(p) for p in file.parents][0:-1] if p in skip_subfiles_of_dir]
            ):
                title = self._read_markdown_title(file=file)
                nav.append({title: str(file.relative_to(relative_to))})
            elif (
                file.is_dir()
                and self.blog_config is not None
                and str(file).endswith(self.blog_config.blog_dir)
            ):
                nav.append({file.stem: str(file.relative_to(relative_to))})
        return nav

    @event_priority(100)  # Run before any other plugins
    def on_config(self, config: MkDocsConfig) -> Optional[Config]:
        from mkdocs_publisher.blog.config import BlogPluginConfig
        from mkdocs_publisher.blog.plugin import BlogPlugin

        self.blog_config = cast(
            BlogPluginConfig,
            _utils.get_plugin_config(
                plugin=BlogPlugin(),
                config_file_path=str(self.config.config_file_path),
                yaml_config_key="pub-blog",
            ),
        )
        if self.blog_config is not None:
            blog_dir = str(Path(config.docs_dir) / Path(self.blog_config.blog_dir))
            if blog_dir not in self.config.skip_dir:
                self.config.skip_dir.append(blog_dir)

        skip_subfiles_of_dir = [str(Path(f)) for f in self.config.skip_dir]
        nav = self._iterate_dir(
            directory=Path(config.docs_dir),
            skip_subfiles_of_dir=skip_subfiles_of_dir,
            relative_to=Path(config.docs_dir),
        )
        config.nav = nav

        return config

    def _iterate_nav(self, items):
        nav = []
        for item in items:
            if (
                isinstance(item, Page)
                and Path(item.file.src_path).parts[-1] != self.config.meta_file_name
            ):
                nav.append(item)
            elif isinstance(item, Section):
                item.children = self._iterate_nav(items=item.children)
                nav.append(item)
            elif isinstance(item, Link) and item.url.split("/")[-1] != self.config.meta_file_name:
                nav.append(item)
        return nav

    def on_nav(
        self, nav: Navigation, *, config: MkDocsConfig, files: Files
    ) -> Optional[Navigation]:

        nav.items = self._iterate_nav(items=nav.items)

        return nav

    # @event_priority(100)  # Run before any other plugins
    def on_files(self, files: Files, *, config: MkDocsConfig) -> Optional[Files]:
        if self.config.remove_sort_prefix_from_slug:
            prefix = rf"^[0-9]+{self.config.sort_prefix_delimiter}"
            new_files = Files([])
            for file in files:
                file.dest_path = str(
                    Path(*[re.sub(prefix, "", part) for part in Path(file.dest_path).parts])
                )
                file.url = str(Path(*[re.sub(prefix, "", part) for part in Path(file.url).parts]))
                abs_dest_path = Path(
                    *[
                        re.sub(prefix, "", part)
                        for part in Path(file.abs_dest_path).relative_to(config.site_dir).parts
                    ]
                )
                file.abs_dest_path = str(Path(config.site_dir) / abs_dest_path)
                if Path(file.src_path).parts[-1] != self.config.meta_file_name:
                    new_files.append(file)
            return new_files
        return files



import importlib.resources
import logging
from dataclasses import fields
from typing import cast

import yaml

from mkdocs_publisher.blog import lang as lang_resources
from mkdocs_publisher.blog.config import BlogPluginConfig
from mkdocs_publisher.blog.structures import Translation

log = logging.getLogger("mkdocs.plugins.publisher.blog")


class Translate:
    def __init__(self, config: BlogPluginConfig):
        self._config: BlogPluginConfig = config
        self._lang: str = self._config.lang
        self._translation: Translation = cast(Translation, None)

        self._read_lang()

    def _read_lang(self):
        try:
            lang_yaml = importlib.resources.read_text(lang_resources, f"{self._config.lang}.yaml")
        except FileNotFoundError:
            log.warning(
                f"There is no translation for '{self._config.lang}' language, "
                f"so default language ('en') will be used"
            )
            lang_yaml = importlib.resources.read_text(
                lang_resources, f"{self._config.lang.default}.yaml"  # type: ignore
            )
        translation_yaml_data = yaml.safe_load(lang_yaml)
        translation_keys = [f.name for f in fields(Translation)]
        translation_data = {
            k: v for k, v in translation_yaml_data.items() if k in translation_keys
        }
        # Inject overrides from mkdocs.yml config
        for key in translation_keys:
            value = self._config.translation.get(key, None)
            if value is not None:
                translation_data[key] = value
        self._translation = Translation(**translation_data)

    @property
    def translation(self) -> Translation:
        return self._translation



import importlib
import importlib.resources
import importlib.util
import logging
from collections import OrderedDict
from datetime import datetime
from pathlib import Path
from typing import Any
from typing import Dict
from typing import Optional
from typing import cast

from mkdocs.config import Config
from mkdocs.config.defaults import MkDocsConfig
from mkdocs.plugins import BasePlugin
from mkdocs.plugins import event_priority
from mkdocs.structure.files import File
from mkdocs.structure.files import Files
from mkdocs.structure.nav import Navigation
from mkdocs.structure.pages import Page

from mkdocs_publisher import _utils
from mkdocs_publisher.blog import creators
from mkdocs_publisher.blog import modifiers
from mkdocs_publisher.blog import parsers
from mkdocs_publisher.blog.config import BlogPluginConfig
from mkdocs_publisher.blog.structures import BlogConfig

log = logging.getLogger("mkdocs.plugins.publisher.blog")


class BlogPlugin(BasePlugin[BlogPluginConfig]):
    def __init__(self):
        self.blog_config = BlogConfig()  # Empty instance
        self.temp_files: Dict[str, Path] = {}

    def on_config(self, config: MkDocsConfig) -> Config:

        # Initialization of all the values
        self.blog_config.parse_configs(mkdocs_config=config, plugin_config=self.config)

        # New config navigation
        config_nav = OrderedDict()
        parsers.parse_markdown_files(
            blog_config=self.blog_config,
            config_nav=config_nav,
        )

        parsers.create_blog_post_teaser(
            blog_config=self.blog_config,
        )

        creators.create_blog_post_pages(
            blog_config=self.blog_config,
            config_nav=config_nav,
        )

        modifiers.blog_post_nav_sorter(
            blog_config=self.blog_config,
            config_nav=config_nav,
        )

        # Modify nav section
        new_nav = []
        if config.nav is None:
            if self.blog_config.plugin_config.start_page:
                config.nav = [{str(self.blog_config.blog_dir): str(self.blog_config.blog_dir)}]
            else:
                config.nav = []
        for n in cast(list, config.nav):
            if str(self.blog_config.blog_dir) in n.values():
                for k, v in config_nav.items():
                    new_nav.append({k: v})
            else:
                new_nav.append(n)
        config.nav = new_nav

        return config

    def on_nav(self, nav: Navigation, config: MkDocsConfig, files: Files) -> Navigation:

        modifiers.blog_post_nav_remove(blog_config=self.blog_config, nav=nav)

        return nav

    def on_files(self, files: Files, config: MkDocsConfig) -> Files:

        creators.create_blog_files(blog_config=self.blog_config, files=files)

        new_files = modifiers.blog_post_slug_modifier(
            blog_config=self.blog_config,
            files=files,
        )

        # TODO: add as _utils and split into blog specific path
        config.extra_css.append("assets/stylesheets/blog.min.css")

        with importlib.resources.path(
            importlib.import_module("mkdocs_publisher._extra"), "__init__.py"
        ) as extra_path:
            s = importlib.import_module("mkdocs_publisher._extra.assets.stylesheets")
            with importlib.resources.path(s, "blog.min.css") as blog_stylesheets:
                new_files.append(
                    File(
                        path=str(blog_stylesheets.relative_to(extra_path.parent)),
                        src_dir=str(extra_path.parent),
                        dest_dir=str(self.blog_config.site_dir),
                        use_directory_urls=self.blog_config.mkdocs_config.use_directory_urls,
                    )
                )

        return new_files

    def on_page_markdown(self, markdown: str, *, page: Page, config: MkDocsConfig, files: Files):
        # Modify page update date
        # TODO: move to meta-apply plugin
        # TODO: move date format to config
        update_date: datetime = page.meta.get(
            "update", page.meta.get("date", datetime.strptime(page.update_date, "%Y-%m-%d"))
        )
        page.update_date = update_date.strftime("%Y-%m-%d")

    @event_priority(-100)  # Run after all other plugins
    def on_page_context(
        self, context: Dict[str, Any], *, page: Page, config: MkDocsConfig, nav: Navigation
    ) -> Optional[Dict[str, Any]]:

        modifiers.blog_post_nav_next_prev_change(blog_config=self.blog_config, page=page)
        return context

    @event_priority(-100)  # Run after all other plugins
    def on_build_error(self, error: Exception) -> None:

        _utils.remove_dir(directory=self.blog_config.temp_dir)

    @event_priority(-100)  # Run after all other plugins
    def on_shutdown(self) -> None:

        _utils.remove_dir(directory=self.blog_config.temp_dir)



from dataclasses import asdict
from dataclasses import dataclass
from dataclasses import field
from datetime import datetime
from pathlib import Path
from typing import Dict
from typing import List
from typing import Optional
from typing import cast

from mkdocs.config.defaults import MkDocsConfig

from mkdocs_publisher import _utils
from mkdocs_publisher.auto_nav.config import AutoNavPluginConfig
from mkdocs_publisher.auto_nav.plugin import AutoNavPlugin
from mkdocs_publisher.blog.config import BlogPluginConfig


@dataclass
class BlogPost:
    """Single blog post data container"""

    title: str
    date: datetime
    path: Optional[str]
    content: Optional[str]
    tags: Optional[List[str]]
    categories: Optional[List[str]]
    slug: Optional[str] = None
    teaser: str = ""
    is_teaser: bool = False

    @property
    def as_dict(self) -> dict:
        return asdict(self)


@dataclass
class Translation:
    teaser_link_text: str
    blog_page_title: str
    blog_navigation_name: str
    recent_blog_posts_navigation_name: str
    archive_page_title: str
    archive_navigation_name: str
    categories_page_title: str
    categories_navigation_name: str
    tags_page_title: str
    tags_navigation_name: str
    newer_posts: str
    older_posts: str

    @property
    def as_dict(self) -> dict:
        return asdict(self)


@dataclass
class BlogConfig:
    mkdocs_config: MkDocsConfig = field(init=False)
    plugin_config: BlogPluginConfig = field(init=False)
    auto_nav_config: Optional[AutoNavPluginConfig] = field(init=False, default=None)
    translation: Translation = field(init=False)
    temp_dir: Path = field(init=False)
    docs_dir: Path = field(init=False)
    blog_dir: Path = field(init=False)
    site_dir: Path = field(init=False)
    blog_posts: Dict[datetime, BlogPost] = field(init=False, default_factory=lambda: dict())
    temp_files: Dict[str, Path] = field(init=False, default_factory=lambda: dict())

    def parse_configs(self, mkdocs_config: MkDocsConfig, plugin_config: BlogPluginConfig):
        from mkdocs_publisher.blog.translate import Translate

        self.mkdocs_config = mkdocs_config
        self.plugin_config = plugin_config
        self.auto_nav_config = cast(
            AutoNavPluginConfig,
            _utils.get_plugin_config(
                plugin=AutoNavPlugin(),
                config_file_path=cast(str, plugin_config.config_file_path),
                yaml_config_key="pub-auto-nav",
            ),
        )
        self.temp_dir = Path(plugin_config.temp_dir)
        self.docs_dir = Path(mkdocs_config.docs_dir)
        self.blog_dir = Path(plugin_config.blog_dir)
        self.site_dir = Path(mkdocs_config.site_dir)
        self.translation = Translate(config=plugin_config).translation

        self.create_dirs()

    def create_dirs(self):
        self.temp_dir.mkdir(exist_ok=True)



from pathlib import Path

import click
import frontmatter

# from _utils import blog_in_cfg
# from _utils import mkdocs_cfg


@click.group
def group():
    pass


@group.command
def create():
    post = frontmatter.Post(content="")
    post["date"] = "2022-01-02 19:22:23"
    post["tags"] = "some, any, none"
    post["category"] = "none"

    post_file_path = Path.cwd() / "docs/posts/temp.md"
    with open(post_file_path, "wb") as post_file:
        frontmatter.dump(post, fd=post_file)


# @group.command
# def temp(mkdocs_cfg=mkdocs_cfg(), blogger_cfg=blogger_cfg()):
#     print(blogger_cfg.images_dir)



import logging
import subprocess
from hashlib import md5
from pathlib import Path
from typing import List
from typing import Optional
from typing import cast
from uuid import uuid4

from mkdocs import utils
from mkdocs.config import Config
from mkdocs.plugins import BasePlugin

log = logging.getLogger("mkdocs.plugins.publisher.common")


def get_plugin_config(
    plugin: BasePlugin, config_file_path: str, yaml_config_key: str
) -> Optional[Config]:
    options = {}
    yaml_config_file = cast(dict, utils.yaml_load(open(config_file_path, "rb")))
    plugin_config_found = False
    for plugin_config in cast(list, yaml_config_file.get("plugins")):
        if isinstance(plugin_config, dict) and list(plugin_config.keys())[0] == yaml_config_key:
            plugin_config_found = True
            options = list(plugin_config.values())[0]
        elif isinstance(plugin_config, str) and plugin_config == yaml_config_key:
            plugin_config_found = True
    plugin.load_config(options=options, config_file_path=config_file_path)

    return plugin.config if plugin_config_found else None


def run_subprocess(cmd) -> int:
    cmd = [arg for arg in cmd if arg is not None]
    log.debug(f"Run cmd: {' '.join(cmd)}")
    call_output = subprocess.run(cmd)
    return call_output.returncode


def remove_dir(directory: Path):
    for file in directory.iterdir():
        if file.is_dir():
            remove_dir(directory=file)
            file.rmdir()
        else:
            file.unlink(missing_ok=True)


def calculate_file_hash(file: Path, block_size: int = 8192) -> str:
    with open(file, "rb") as binary_file:
        hash = md5()
        while chunk := binary_file.read(block_size):
            hash.update(chunk)
        return hash.hexdigest()


def list_files(
    directory: Path, extensions: Optional[List[str]] = None, relative_to: Optional[Path] = None
) -> List[Path]:
    files_list: List[Path] = []
    for file in directory.iterdir():
        if file.is_dir():
            files_list.extend(
                list_files(directory=file, extensions=extensions, relative_to=relative_to)
            )
        elif str(file.suffix).lower() in extensions or extensions is None:  # type: ignore
            files_list.append(file if relative_to is None else file.relative_to(relative_to))
    return files_list


def get_hashed_file_name(file: Path) -> Path:
    return Path(file.with_name(f"{str(uuid4()).replace('-', '')}{str(file.suffix).lower()}").name)



from mkdocs.config import config_options as option
from mkdocs.config.base import Config


class AutoNavPluginConfig(Config):
    skip_dir = option.ListOfPaths(default=[])
    meta_file_name = option.Type(str, default="README.md")
    sort_prefix_delimiter = option.Type(str, default="_")
    remove_sort_prefix_from_slug = option.Type(bool, default=True)



import logging
from multiprocessing import cpu_count
from pathlib import Path
from typing import Dict
from typing import cast

import yaml
from mkdocs.config.defaults import MkDocsConfig
from mkdocs.plugins import BasePlugin
from mkdocs.plugins import event_priority

from mkdocs_publisher.minifier import minifiers
from mkdocs_publisher.minifier.base import CachedFile
from mkdocs_publisher.minifier.config import MinifierConfig

log = logging.getLogger("mkdocs.plugins.publisher.minifier")


class MinifierPlugin(BasePlugin[MinifierConfig]):
    @event_priority(-100)  # Run after all other plugins
    def on_post_build(self, *, config: MkDocsConfig) -> None:

        Path(self.config.cache_dir).mkdir(exist_ok=True)

        # TODO: Add path to tools checker
        cached_files: Dict[str, CachedFile] = {}

        if self.config.threads == 0:
            self.config.threads = int(cpu_count())
        log.info(f"Threads used for minifiers: {self.config.threads}")

        cached_files_list: Path = Path(self.config.cache_dir) / self.config.cache_file
        if cached_files_list.exists():
            try:
                with open(cached_files_list, "r") as yaml_file:
                    cached_files = yaml.safe_load(yaml_file)
                    for file_path, cached_file in cached_files.items():
                        cached_files[file_path] = CachedFile(**cast(dict, cached_file))
            except (yaml.YAMLError, AttributeError) as e:
                log.warning(f"File '{cached_files_list}' corrupted. Rebuilding cache.")
                log.debug(e)

        if self.config.png.enabled:
            minifiers.PngMinifier(
                plugin_config=self.config, mkdocs_config=config, cached_files=cached_files
            )()

        if self.config.jpg.enabled:
            minifiers.JpgMinifier(
                plugin_config=self.config, mkdocs_config=config, cached_files=cached_files
            )()

        if self.config.svg.enabled:
            minifiers.SvgMinifier(
                plugin_config=self.config, mkdocs_config=config, cached_files=cached_files
            )()

        if self.config.html.enabled:
            minifiers.HtmlMinifier(
                plugin_config=self.config, mkdocs_config=config, cached_files=cached_files
            )()

        if self.config.css.enabled:
            minifiers.CssMinifier(
                plugin_config=self.config, mkdocs_config=config, cached_files=cached_files
            )()

        if self.config.js.enabled:
            minifiers.JsMinifier(
                plugin_config=self.config, mkdocs_config=config, cached_files=cached_files
            )()

        cached_files_for_yaml = {}
        for file_name, cached_file in cached_files.items():
            # TODO: Cleanup yaml from unused/deleted files and delete those that are unused
            cached_file = {k: str(v) for k, v in cached_file.as_dict.items()}
            cached_files_for_yaml[file_name] = cached_file

        with open(cached_files_list, "w") as yaml_file:
            yaml.safe_dump(cached_files_for_yaml, yaml_file)



import logging
from typing import Optional

from mkdocs_publisher import _utils
from mkdocs_publisher.minifier.base import BaseMinifier
from mkdocs_publisher.minifier.base import CachedFile

log = logging.getLogger("mkdocs.plugins.publisher.minifier")


class PngMinifier(BaseMinifier):
    extensions = [".png"]

    def minifier(self, cached_file: CachedFile) -> Optional[CachedFile]:
        minify_options = self._plugin_config.png
        try:
            input_file = self._mkdocs_config.site_dir / cached_file.original_file_path
            output_file = self._plugin_config.cache_dir / cached_file.cached_file_name

            if minify_options.pngquant_enabled:
                pngquant_cmd = [
                    minify_options.pngquant_path,
                    "--force",
                    "--strip" if minify_options.strip else None,
                    "--quality" if int(minify_options.pngquant_quality) > 0 else None,
                    minify_options.pngquant_quality
                    if int(minify_options.pngquant_quality) > 0
                    else None,
                    "--speed",
                    minify_options.pngquant_speed,
                    "--output",
                    str(output_file),
                    str(input_file),
                ]
                if _utils.run_subprocess(cmd=pngquant_cmd) != 0:
                    output_file.unlink(missing_ok=True)
                    return None

            if minify_options.oxipng_enabled:
                oxipng_cmd = [
                    minify_options.oxipng_path,
                    "--quiet",
                    "--force",
                    "--strip" if minify_options.strip else None,
                    "all" if minify_options.strip else None,
                    "--zopfli" if minify_options.oxipng_max_compression else None,
                    "--out" if not minify_options.pngquant_enabled else None,
                    str(output_file) if not minify_options.pngquant_enabled else None,
                    str(input_file) if not minify_options.pngquant_enabled else str(output_file),
                ]
                if _utils.run_subprocess(cmd=oxipng_cmd) != 0:
                    output_file.unlink(missing_ok=True)
                    return None

                return cached_file

        except FileNotFoundError as e:
            log.warning(e)

        return None


class JpgMinifier(BaseMinifier):
    extensions = [".jpg", ".jpeg"]

    def minifier(self, cached_file: CachedFile) -> Optional[CachedFile]:
        minify_options = self._plugin_config.jpg
        try:
            input_file = self._mkdocs_config.site_dir / cached_file.original_file_path
            output_file = self._plugin_config.cache_dir / cached_file.cached_file_name

            djpg_cmd = [
                minify_options.djpeg_path,
                "-targa",
                "-outfile",
                str(output_file.with_suffix(".tga")),
                str(input_file),
            ]
            if _utils.run_subprocess(cmd=djpg_cmd) != 0:
                output_file.with_suffix(".tga").unlink(missing_ok=True)
                return None

            cjpg_cmd = [
                minify_options.cjpeg_path,
                "-targa",
                "-smooth" if int(minify_options.smooth) > 0 else None,
                minify_options.smooth if int(minify_options.smooth) > 0 else None,
                "-quality" if int(minify_options.quality) > 0 else None,
                minify_options.quality if int(minify_options.quality) > 0 else None,
                "-outfile",
                str(output_file.with_suffix(".jpg_")),
                str(output_file.with_suffix(".tga")),
            ]
            if _utils.run_subprocess(cmd=cjpg_cmd) != 0:
                output_file.with_suffix(".jpg_").unlink(missing_ok=True)
                return None
            output_file.with_suffix(".tga").unlink(missing_ok=True)

            jpegtran_cmd = [
                minify_options.jpegtran_path,
                "-copy",
                minify_options.copy_meta,
                "-progressive" if minify_options.progressive else None,
                "-optimise" if minify_options.optimise else None,
                "-outfile",
                str(output_file),
                str(output_file.with_suffix(".jpg_")),
            ]
            if _utils.run_subprocess(cmd=jpegtran_cmd) != 0:
                output_file.unlink(missing_ok=True)
                return None
            output_file.with_suffix(".jpg_").unlink(missing_ok=True)

            return cached_file

        except FileNotFoundError as e:
            log.warning(e)

        return None


class SvgMinifier(BaseMinifier):
    extensions = [".svg"]

    def minifier(self, cached_file: CachedFile) -> Optional[CachedFile]:
        minify_options = self._plugin_config.svg
        try:
            input_file = self._mkdocs_config.site_dir / cached_file.original_file_path
            output_file = self._plugin_config.cache_dir / cached_file.cached_file_name

            svgo_cmd = [
                minify_options.svgo_path,
                "--multipass" if minify_options.multipass else None,
                "--quiet",
                "--output",
                str(output_file),
                "--input",
                str(input_file),
            ]
            if _utils.run_subprocess(cmd=svgo_cmd) != 0:
                output_file.unlink(missing_ok=True)
                return None

            return cached_file

        except FileNotFoundError as e:
            log.warning(e)

        return None


class HtmlMinifier(BaseMinifier):
    extensions = [".htm", ".html"]

    def minifier(self, cached_file: CachedFile) -> Optional[CachedFile]:
        minify_options = self._plugin_config.html
        try:
            input_file = self._mkdocs_config.site_dir / cached_file.original_file_path
            output_file = self._plugin_config.cache_dir / cached_file.cached_file_name

            html_minifier_cmd = [
                minify_options.html_minifier_path,
                "--case-sensitive" if minify_options.case_sensitive else None,
                "--minify-css" if minify_options.minify_css else None,
                "--minify-js" if minify_options.minify_js else None,
                "--remove-comments" if minify_options.remove_comments else None,
                "--remove-tag-whitespace" if minify_options.remove_tag_whitespace else None,
                "--collapse-whitespace" if minify_options.collapse_whitespace else None,
                "--conservative-collapse" if minify_options.conservative_collapse else None,
                "--collapse-boolean-attributes"
                if minify_options.collapse_boolean_attributes
                else None,
                "--preserve-line-breaks" if minify_options.preserve_line_breaks else None,
                "--max-line-length" if int(minify_options.max_line_length) > 0 else None,
                minify_options.max_line_length
                if int(minify_options.max_line_length) > 0
                else None,
                "--sort-attributes" if minify_options.sort_attributes else None,
                "--sort-class-name" if minify_options.sort_class_name else None,
                "--output",
                str(output_file),
                str(input_file),
            ]
            if _utils.run_subprocess(cmd=html_minifier_cmd) != 0:
                output_file.unlink(missing_ok=True)
                return None

            return cached_file

        except FileNotFoundError as e:
            log.warning(e)

        return None


class CssMinifier(BaseMinifier):
    extensions = [".css"]

    def minifier(self, cached_file: CachedFile) -> Optional[CachedFile]:
        minify_options = self._plugin_config.css
        try:
            input_file = self._mkdocs_config.site_dir / cached_file.original_file_path
            output_file = self._plugin_config.cache_dir / cached_file.cached_file_name

            if ".min" in input_file.suffixes and minify_options.skip_minified:
                return None
            css_minifier_cmd = [
                minify_options.postcss_path,
                "--no-map",
                "--use",
                "cssnano",
                "postcss-svgo",
                "--output",
                str(output_file),
                str(input_file),
            ]
            if _utils.run_subprocess(cmd=css_minifier_cmd) != 0:
                output_file.unlink(missing_ok=True)
                return None

            return cached_file

        except FileNotFoundError as e:
            log.warning(e)

        return None


class JsMinifier(BaseMinifier):
    extensions = [".js"]

    def minifier(self, cached_file: CachedFile) -> Optional[CachedFile]:
        minify_options = self._plugin_config.js
        try:
            input_file = self._mkdocs_config.site_dir / cached_file.original_file_path
            output_file = self._plugin_config.cache_dir / cached_file.cached_file_name

            if ".min" in input_file.suffixes and minify_options.skip_minified:
                return None
            js_minifier_cmd = [
                minify_options.uglifyjs_path,
                "--compress",
                "--webkit",
                "--output",
                str(output_file),
                str(input_file),
            ]
            if _utils.run_subprocess(cmd=js_minifier_cmd) != 0:
                output_file.unlink(missing_ok=True)
                return None

            return cached_file

        except FileNotFoundError as e:
            log.warning(e)

        return None



from pathlib import Path
from typing import cast

from mkdocs import config
from mkdocs import utils
from mkdocs.config.defaults import MkDocsConfig
from mkdocs.plugins import BasePlugin

from blog.config import BlogPluginConfig
from blog.plugin import BlogPlugin


def mkdocs_cfg() -> MkDocsConfig:
    return config.load_config(config_file=open(Path.cwd() / "mkdocs.yml", "rb"))


def blog_in_cfg() -> BlogPluginConfig:
    plugin = BlogPlugin()

    options = {}
    yaml_file = cast(dict, utils.yaml_load(open(Path.cwd() / "mkdocs.yml", "rb")))
    for plugin in cast(list, yaml_file.get("plugins")):
        if isinstance(plugin, dict) and list(plugin.keys())[0] == "blog":
            options = list(plugin.values())[0]

    plugin: BasePlugin
    plugin.load_config(options=options, config_file_path=str(Path.cwd() / "mkdocs.yml"))

    return plugin.config



import importlib.resources
import logging
from collections import OrderedDict
from datetime import datetime
from pathlib import Path
from typing import Dict
from typing import cast

import frontmatter
import jinja2
from mkdocs.structure.files import File
from mkdocs.structure.files import Files

from mkdocs_publisher._extra.assets import templates
from mkdocs_publisher.blog.structures import BlogConfig

log = logging.getLogger("mkdocs.plugins.publisher.blog")


def create_blog_files(
    blog_config: BlogConfig,
    files: Files,
):
    for temp_file in blog_config.temp_files.values():
        try:
            parts = [p for p in Path(temp_file).relative_to(blog_config.temp_dir).parts]
            if parts[0] == blog_config.plugin_config.blog_dir:
                parts[0] = blog_config.plugin_config.slug
            file = File(
                path=str(Path(*parts)),
                src_dir=str(blog_config.temp_dir),
                dest_dir=str(blog_config.site_dir),
                use_directory_urls=blog_config.mkdocs_config.use_directory_urls,
            )
            files.append(file)
        except ValueError:
            pass


def create_blog_post_pages(
    blog_config: BlogConfig,
    config_nav: OrderedDict,
) -> None:
    """Create blog posts index files."""

    log.info("Creating blog posts index files")

    posts_chunks: Dict[str, list] = {}
    archive_chunks: Dict[str, list] = {}
    categories_chunks: Dict[str, list] = {}
    tags_chunks: Dict[str, list] = {}

    # Build post index pages
    for index, date in enumerate(sorted(blog_config.blog_posts, reverse=True)):
        index = (
            "index"
            if blog_config.plugin_config.start_page
            and index < blog_config.plugin_config.posts_per_page
            else f"index-{str(index//blog_config.plugin_config.posts_per_page)}"
        )
        # if not blog_config.plugin_config.start_page and index == "index-0":
        #     index = blog_config.plugin_config.slug
        if index not in posts_chunks:
            posts_chunks[index] = []
        posts_chunks[index].append(blog_config.blog_posts[date])

    # Build archive, category and tag pages
    for date in sorted(blog_config.blog_posts, reverse=True):
        date: datetime
        year = str(date.year)
        if year not in archive_chunks:
            archive_chunks[year] = []
        archive_chunks[year].append(blog_config.blog_posts[date].as_dict)

        categories = cast(str, blog_config.blog_posts[date].categories)
        for category in categories:
            if category not in categories_chunks:
                categories_chunks[category] = []
            categories_chunks[category].append(blog_config.blog_posts[date].as_dict)

        tags = cast(list, blog_config.blog_posts[date].tags)
        for tag in tags:
            if tag not in tags_chunks:
                tag = tag.replace("/", "-")
                tags_chunks[tag] = []
            tags_chunks[tag].append(blog_config.blog_posts[date].as_dict)

    # Reorder categories alphabetically
    categories_chunks = {cat: categories_chunks[cat] for cat in sorted(categories_chunks)}

    # Reorder tags alphabetically
    tags_chunks = {tag: tags_chunks[tag] for tag in sorted(tags_chunks)}
    config_nav[blog_config.translation.blog_navigation_name] = {}

    for key, single_posts_chunk in posts_chunks.items():
        file_name = f"{key}.md"
        file_path = blog_config.temp_dir / file_name

        _render_and_write_page(
            single_posts_chunk=single_posts_chunk,
            file_path=file_path,
            blog_config=blog_config,
            page_title=blog_config.translation.blog_page_title,
        )

        blog_config.temp_files[f"{blog_config.translation.blog_navigation_name}/{key}"] = file_path
        log.debug(f"Creating blog post chunk file: {file_path}")

        config_nav[blog_config.translation.blog_navigation_name][key] = f"{file_name}"

    _create_pages(
        blog_config=blog_config,
        posts_chunks=archive_chunks,
        config_nav=config_nav[blog_config.translation.blog_navigation_name],
        sub_dir=Path(blog_config.plugin_config.archive_subdir),
        navigation_name=blog_config.translation.archive_navigation_name,
        page_title=blog_config.translation.archive_page_title,
    )

    _create_pages(
        blog_config=blog_config,
        posts_chunks=categories_chunks,
        config_nav=config_nav[blog_config.translation.blog_navigation_name],
        sub_dir=Path(blog_config.plugin_config.categories_subdir),
        navigation_name=blog_config.translation.categories_navigation_name,
        page_title=blog_config.translation.categories_page_title,
    )

    _create_pages(
        blog_config=blog_config,
        posts_chunks=tags_chunks,
        config_nav=config_nav[blog_config.translation.blog_navigation_name],
        sub_dir=Path(blog_config.plugin_config.tags_subdir),
        navigation_name=blog_config.translation.tags_navigation_name,
        page_title=blog_config.translation.tags_page_title,
    )


def _create_pages(
    blog_config: BlogConfig,
    posts_chunks: Dict[str, list],
    config_nav: OrderedDict,
    sub_dir: Path,
    navigation_name: str,
    page_title: str,
):
    config_nav[navigation_name] = {}

    # pages_dir = blog_config.docs_dir / blog_config.blog_dir / sub_dir

    blog_temp_dir = blog_config.temp_dir / blog_config.plugin_config.slug
    blog_temp_dir.mkdir(exist_ok=True)

    pages_dir = blog_temp_dir / sub_dir
    pages_dir.mkdir(exist_ok=True)

    for key, single_posts_chunk in posts_chunks.items():
        file_name = f"{key}.md"
        file_path = pages_dir / file_name
        _render_and_write_page(
            single_posts_chunk=single_posts_chunk,
            file_path=file_path,
            blog_config=blog_config,
            page_title=f"{page_title} - {key}",
        )

        blog_config.temp_files[f"{sub_dir}/{key}"] = file_path
        log.debug(f"Creating blog post chunk file: {file_path}")

        config_nav[navigation_name][
            key
        ] = f"{blog_config.plugin_config.slug}/{sub_dir}/{file_name}"


def _render_and_write_page(
    single_posts_chunk: list,
    file_path: Path,
    blog_config: BlogConfig,
    page_title: str,
):
    # TODO: Add templates from override
    # templates = jinja2.Environment(loader=jinja2.FileSystemLoader("templates/"))
    # print(templates.list_templates())
    # template = templates.get_template("index.html")
    index_template = importlib.resources.read_text(templates, "posts-list.html")
    context = {
        "posts": single_posts_chunk,
        "site_url": str(blog_config.mkdocs_config.site_url),
        "config": blog_config.plugin_config,
        "translation": blog_config.translation,
    }
    template = jinja2.Environment(loader=jinja2.BaseLoader()).from_string(index_template)

    page = frontmatter.Post(content=template.render(context))
    page["title"] = page_title

    with open(file_path, mode="wb") as teasers_index:
        frontmatter.dump(page, teasers_index)



import click

# from src.cli import post


@click.group
def app():
    """Publisher plugin for MkDocs (by Maciej 'maQ' Kusz)
    More information can be fount at: https://github.com/mkusz/mkdocs-publisher
    """
    pass


# app.add_command(cmd=post.group, name="post")


if __name__ == "__main__":
    app()



import logging
from collections import OrderedDict
from copy import deepcopy
from typing import cast

from mkdocs.structure.files import Files
from mkdocs.structure.nav import Navigation
from mkdocs.structure.nav import Section
from mkdocs.structure.pages import Page

from mkdocs_publisher.blog.structures import BlogConfig

log = logging.getLogger("mkdocs.plugins.publisher.blog")


def blog_post_slug_modifier(blog_config: BlogConfig, files: Files) -> Files:
    """Modify File object destination file paths and url to defined blog post slug."""

    blog_posts = {post.path: post for post in blog_config.blog_posts.values()}  # type: ignore
    new_files = Files([])

    log.info("Modify blog posts url addresses based on slug")
    for file in files:
        if file.src_uri in blog_posts and blog_posts[file.src_uri].slug is not None:
            url = file.url.split("/")
            url[-1] = blog_posts[file.src_uri].slug  # type: ignore
            file.url = f"{'/'.join(url)}/"
            file.name = str(blog_posts[file.src_uri].slug)
            url.append(file.dest_uri.split("/")[-1])
            file.dest_uri = "/".join(url)
            file.abs_dest_path = str(blog_config.site_dir / file.dest_uri)
            log.debug(f"Blog post: {blog_posts[file.src_uri].title} url is: {file.url}")
        new_files.append(file)

    return new_files


def blog_post_nav_sorter(
    blog_config: BlogConfig,
    config_nav: OrderedDict,
):
    """Reorder blog posts in config navigation section from newest to oldest."""

    log.info("Reorder blog posts from newest to oldest")
    config_nav["_blog_posts_"] = {
        blog_config.blog_posts[date].title: blog_config.blog_posts[date].path
        for date in sorted(blog_config.blog_posts, reverse=True)
    }


def blog_post_nav_remove(
    blog_config: BlogConfig,
    nav: Navigation,
) -> None:
    """Remove blog posts pages, subindexes and section from direct navigation."""

    log.info("Removing blog posts pages and section from direct navigation")
    nav.items = [
        i for i in nav.items if not (isinstance(i, Section) and i.title.lower() == "_blog_posts_")
    ]
    log.info("Removing blog sub index pages from navigation menu")
    for item in nav.items:
        if (
            isinstance(item, Section)
            and item.title == blog_config.translation.blog_navigation_name
        ):
            children = []
            for section_item in item.children:
                if not (
                    isinstance(section_item, Page) and str(section_item.title).startswith("index-")
                ):
                    children.append(section_item)
                if (
                    isinstance(section_item, Page)
                    and not blog_config.plugin_config.start_page
                    and str(section_item.title).startswith("index-0")
                ):
                    children.append(section_item)
            item.children = children


def blog_post_nav_next_prev_change(blog_config: BlogConfig, page: Page):
    """Change blog post next/prev navigation"""

    if (blog_config.plugin_config.start_page and page.title == "index") or (
        not blog_config.plugin_config.start_page and page.title == "index-0"
    ):
        page.title = blog_config.translation.recent_blog_posts_navigation_name
        if page.next_page is not None and str(page.next_page.title).startswith("index-"):
            next_page_copy = cast(Page, deepcopy(page.next_page))
            next_page_copy.title = blog_config.translation.older_posts
            page.next_page = next_page_copy
    if str(page.title).startswith("index") or (
        page.previous_page is not None and str(page.previous_page.title).startswith("index")
    ):
        previous_page_copy = cast(Page, deepcopy(page.previous_page))
        previous_page_copy.title = blog_config.translation.newer_posts
        page.previous_page = previous_page_copy
        if page.next_page is not None and str(page.next_page.title).startswith("index-"):
            next_page_copy = cast(Page, deepcopy(page.next_page))
            next_page_copy.title = blog_config.translation.older_posts
            page.next_page = next_page_copy



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['mkdocs_publisher',
 'mkdocs_publisher._cli',
 'mkdocs_publisher._extra',
 'mkdocs_publisher._extra.assets',
 'mkdocs_publisher._extra.assets.stylesheets',
 'mkdocs_publisher._extra.assets.templates',
 'mkdocs_publisher.auto_nav',
 'mkdocs_publisher.blog',
 'mkdocs_publisher.blog.lang',
 'mkdocs_publisher.minifier']

package_data = \
{'': ['*']}

install_requires = \
['mkdocs-material>=9.1.3,<10.0.0',
 'mkdocs>=1.4.2,<2.0.0',
 'pymdown-extensions>=9.10,<10.0',
 'python-frontmatter>=1.0.0,<2.0.0']

entry_points = \
{'console_scripts': ['publisher = mkdocs_publisher._cli.publisher:app'],
 'mkdocs.plugins': ['pub-auto-nav = '
                    'mkdocs_publisher.auto_nav.plugin:AutoNavPlugin',
                    'pub-blog = mkdocs_publisher.blog.plugin:BlogPlugin',
                    'pub-minifier = '
                    'mkdocs_publisher.minifier.plugin:MinifierPlugin']}

setup_kwargs = {
    'name': 'mkdocs-publisher',
    'version': '0.4.1',
    'description': 'Publishing plugins for MkDocs',
    'long_description': "---\ndate: 2023-02-28 12:30:18\nupdate: 2023-03-28 12:15:52\n---\n# Publisher plugin for MkDocs\n\n[![![PyPI version](https://img.shields.io/pypi/v/mkdocs-publisher?logo=pypi&style=plastic)](https://pypi.org/project/mkdocs-publisher/)\n[![License type](https://img.shields.io/pypi/l/mkdocs-publisher?logo=pypi&style=plastic)](https://opensource.org/license/bsd-2-clause/)\n[![PyPI Downloads last month](https://img.shields.io/pypi/dm/mkdocs-publisher?logo=pypi&style=plastic)](https://pypistats.org/search/mkdocs-publisher)\n[![Python versions](https://img.shields.io/pypi/pyversions/mkdocs-publisher?logo=python&style=plastic)](https://www.python.org)\n[![GitHub last commit](https://img.shields.io/github/last-commit/mkusz/mkdocs-publisher?logo=github&style=plastic)](https://github.com/mkusz/mkdocs-publisher/commits/main)\n\nPublishing platform plugins for [MkDocs](https://www.mkdocs.org/) that include:\n\n- `pub-auto-nav`– building site navigation right from files (no need for manual definition in config),\n- `pub-blog` – adds blogging capability,\n- `pub-minifier` – file size optimization (good for SEO and overall page size optimization).\n\n## Installation\n\n```commandline\npip install mkdocs-publisher\n```\n\n## Basic usage\n\n> **Note**\n> As a base for any development, [mkdocs-material](https://squidfunk.github.io/mkdocs-material/) theme was used. If you are willing to use any other theme, you may (or may not) face some issues. If this happens, please submit an issue.\n\n> **Warning**\n> Consider this plugin as a beta, so before any use make sure you have a backup of your data.\n\nIf you have found any issue, have an idea for a feature, please submit an issue.\n\n## Features\n\nList of included features (more documentation is needed):\n\n- automatic blog post index page generation with blog post teasers based on delimiter inside a blog post and own template (delimiter can be changed in plugin config in `mkdocs.yaml`),\n- blog post/page update date based on blog post metadata,\n- separate directory for blog post documents with auto-generated separate navigation (blog posts are sorted from newest to oldest),\n- home page set to blog post index with possibility to rename,\n- auto-adding link to full blog post from blog post index file (under each post that has teaser delimiter, if delimiter is not present, then full post is inside post index file, but is preserved in blog post navigation and site map),\n- added sub-pages for blog posts: archive, categories, tags,\n- minification plugin for graphics and documentation files.\n\n## How To\n\nMore detailed information on how to setup, configure and write a blog posts and/or documentation can be found in [documentation](https://mkusz.github.io/mkdocs-publisher/)\n\n## Todo's\n\nA full list of planned developments can be found on [this documentation page](https://mkusz.github.io/mkdocs-publisher/dev/backlog/).\n\n## Version history\n\n### 0.4.1 - 2023-03-28\n\nGeneral:\n\n- fix: links in documentation\n- fix: imports of libraries\n- fix: badges links + new added\n\n### 0.4.0 - 2023-03-28\n\nGeneral:\n\n- changed: project rename\n- added: cross configuration of blog and auto-nav plugins:\n  - blog does not add auto-nav meta files\n  - auto-nav automatically adds blog directory to skipped directories since it will be built by blog\n  - if one of the plugins is not enabled, other is not using its values\n- add: documentation\n\nBlog:\n\n- added: possibility to choose a blog as a starting page with option to define manually blog in nav configuration\n- added: `slug` config option for setting an entire blog's main directory URL\n- changed: internal file structure refactor with new global plugin config (`BlogConfig` class) that will help with further development with small fixes and improvements\n- changed: blog subdirectory navigation creation (entry path needs to be equal to subdirectory name)\n- fixed: live reload infinite loop during `serve` caused by temporary files created and removed in blog directory\n- fixed: navigation is no longer overridden by a blog (if there is no other nav, blog will create on with recent posts as a main page)\n\nMinifier (new plugin):\n\n- added: PNG image minifier (using: pngquant and oxipng)\n- added: JPG image minifier (using: mozjpeg)\n- added: SVG image minifier (using: svgo)\n- added: HTML file minifier (using: html-minifier)\n- added: CSS file minifier (using: postcss with plugins: cssnano, svgo)\n- added: JS file minifier (using: uglifyjs)\n\nAuto-nav (new plugin):\n\n- added: build navigation based on file names\n- added: directory metadata and additional settings can be set in a frontmatter of `*.md` file (default to `README.md`)\n- added: configuration of sort prefix delimiter\n- added: sort prefix removal in URL and site files\n- added: read file title from `title` meta data key\n\n### 0.3.0 - 2023.02.20\n\n- fixed: for wrong directory structure in site-packages after install\n\n### 0.2.0 - 2023.02.19\n\n- added: sub-pages for archive, categories, blog\n- added: configurable blog posts pagination with page navigation\n- added: interface language change: EN and PL (help wanted with more languages)\n- added: possibility to override for all interface text elements\n\n### 0.1.0 - initial release\n\n- added: blog post update date based on metadata\n- added: blog post URL link based on metadata\n- added: blog post tags and categories based on metadata\n- added: support for blog post teaser\n- added: auto generation of blog posts navigation\n",
    'author': "Maciej 'maQ' Kusz",
    'author_email': 'maciej.kusz@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/mkusz/mkdocs-publisher',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)



from mkdocs.config import config_options as option
from mkdocs.config.base import Config


class _MinifierCssConfig(Config):
    enabled = option.Type(bool, default=True)
    postcss_path = option.Type(str, default="postcss")
    skip_minified = option.Type(bool, default=True)


class _MinifierJsConfig(Config):
    enabled = option.Type(bool, default=True)
    uglifyjs_path = option.Type(str, default="uglifyjs")
    skip_minified = option.Type(bool, default=True)


class _MinifierHtmlConfig(Config):
    enabled = option.Type(bool, default=True)
    html_minifier_path = option.Type(str, default="html-minifier")
    case_sensitive = option.Type(bool, default=True)
    minify_css = option.Type(bool, default=True)
    minify_js = option.Type(bool, default=True)
    remove_comments = option.Type(bool, default=True)
    remove_tag_whitespace = option.Type(bool, default=True)
    collapse_whitespace = option.Type(bool, default=True)
    conservative_collapse = option.Type(bool, default=True)
    collapse_boolean_attributes = option.Type(bool, default=True)
    preserve_line_breaks = option.Type(bool, default=True)
    sort_attributes = option.Type(bool, default=True)
    sort_class_name = option.Type(bool, default=True)
    max_line_length = option.Choice(
        [str(i) for i in range(1, 4096)], default="1024"
    )  # 0 - disabled


class _MinifierSvgConfig(Config):
    enabled = option.Type(bool, default=True)
    svgo_path = option.Type(str, default="svgo")
    multipass = option.Type(bool, default=True)


class _MinifierJpgConfig(Config):
    enabled = option.Type(bool, default=True)
    djpeg_path = option.Type(str, default="djpeg")
    cjpeg_path = option.Type(str, default="cjpeg")
    jpegtran_path = option.Type(str, default="jpegtran")
    optimise = option.Type(bool, default=True)
    progressive = option.Type(bool, default=True)
    copy_meta = option.Choice(["none", "comments", "icc", "all"], default="none")
    smooth = option.Choice([str(i) for i in range(0, 101)], default="10")  # 0 - disabled
    quality = option.Choice([str(i) for i in range(0, 101)], default="85")  # 0 - disabled


class _MinifierPngConfig(Config):
    enabled = option.Type(bool, default=True)
    pngquant_enabled = option.Type(bool, default=True)
    pngquant_path = option.Type(str, default="pngquant")
    pngquant_speed = option.Choice([str(i) for i in range(1, 12)], default="1")
    pngquant_quality = option.Choice([str(i) for i in range(0, 101)], default="95")  # 0 - disabled
    oxipng_enabled = option.Type(bool, default=True)
    oxipng_path = option.Type(str, default="oxipng")
    oxipng_max_compression = option.Type(bool, default=True)
    strip = option.Type(bool, default=True)


class MinifierConfig(Config):
    cache_dir = option.Type(str, default=".cache")
    cache_file = option.Type(str, default=".cached_files_list.yml")
    threads = option.Type(int, default=0)  # 0 - default (read from system)

    js: _MinifierJsConfig = option.SubConfig(_MinifierJsConfig)  # type: ignore
    css: _MinifierCssConfig = option.SubConfig(_MinifierCssConfig)  # type: ignore
    jpg: _MinifierJpgConfig = option.SubConfig(_MinifierJpgConfig)  # type: ignore
    png: _MinifierPngConfig = option.SubConfig(_MinifierPngConfig)  # type: ignore
    svg: _MinifierSvgConfig = option.SubConfig(_MinifierSvgConfig)  # type: ignore
    html: _MinifierHtmlConfig = option.SubConfig(_MinifierHtmlConfig)  # type: ignore
